<?php

namespace NoccyLabs\TinyContainer;

trait ContainerTrait
{
    private $services = [];
    private $instances = [];

    public function offsetSet($name, $def)
    {
        $this->services[$name] = $def;
    }

    public function offsetGet($name)
    {
        return $this->services[$name];
    }

    public function offsetExists($name)
    {
        return array_key_exists($name, $this->services);
    }

    public function offsetUnset($name)
    {
        unset($this->instances[$name], $this->services[$name]);
    }

    public function __invoke($name, ...$args)
    {
        if (!array_key_exists($name, $this->instances)) {
            if (!array_key_exists($name, $this->services)) {
                throw new ContainerException("No such service: {$name}");
            }
            if (is_object($this->services[$name])) {
                $this->instances[$name] = $this->services[$name];
            } elseif (is_callable($this->services[$name])) {
                $inst = call_user_func($this->services[$name]);
                if (!is_object($inst)) {
                    throw new ContainerException("Service factory didn't return an object");
                }
                $this->instances[$name] = $inst;
            } elseif (class_exists($this->services[$name])) {
                $this->instances[$name] = new $this->services[$name];
            } else {
                throw new ContainerException("Invalid service definition for {$name}");
            }
        }

        if (is_callable($this->instances[$name])) {
            return call_user_func_array($this->instances[$name], $args);
        } elseif (count($args)>0) {
            throw new ContainerException("Service {$name} is not callable");
        }
        return $this->instances[$name];
    }
}

